/*
 * Copyright 2005 TSS Tsuta
 */
package tss.sc.counter;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Properties;

import javax.swing.JTable;

import tss.sc.gui.table.MyTableModel;
import tss.sc.gui.util.Reporter;
import tss.sc.util.Constants;

/**
 * @author tsuta
 * 
 * JE^[̊NX
 */
public abstract class Counter {

	protected static final int STATEMENT_LINE = 0;

	protected static final int COMMENT_LINE = 1;

	protected static final int BLANK_LINE = 2;

	private static final String ENCODING = "eucjp";

	private static final int BUFFER_SIZE = 4096;

	private static final String PROP_FILE_PATH = "./counter.properties";

	private static Properties prop = null;

	private static HashMap concreteClassMap = new HashMap();

	static {
		Properties prop = new Properties();
		File file = new File(PROP_FILE_PATH);
		if (file.isFile()) {
			try {
				prop.load(new FileInputStream(file));
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		}

		Enumeration enum = prop.propertyNames();
		String key;
		String fqcn;
		while (enum.hasMoreElements()) {
			key = (String) enum.nextElement();
			try {
				fqcn = prop.getProperty(key);
				concreteClassMap.put(key, Class.forName(fqcn).newInstance());
			} catch (Exception e) {
				e.printStackTrace();
				Reporter.report(Constants.REPORT_ERROR, e);
			}
		}
	}

	/**
	 * w̍sɎwt@C̃JEg𔽉f
	 * 
	 * @param table
	 * @param rowIndex
	 * @param file
	 * @throws UnsupportedEncodingException
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public static void countAndReflect(JTable table, int rowIndex, File file)
			throws UnsupportedEncodingException, FileNotFoundException,
			IOException {

		MyTableModel tableModel = (MyTableModel) table.getModel();
		int[] counts = count(file);
		if (counts != null) {
			tableModel.setValueAt(new Integer(counts[0]), rowIndex, 3);
			tableModel.setValueAt(new Integer(counts[1]), rowIndex, 4);
			tableModel.setValueAt(new Integer(counts[2]), rowIndex, 5);
			tableModel.setValueAt(new Integer(counts[3]), rowIndex, 6);
		}
	}

	/**
	 * w̃t@C̃Xe[ggsARgsAsёSsJEg 
	 * ̕тint̔zƂĕԂB <br>
	 * w̃t@CJEg邽߂̃JE^[݂ȂꍇnullԂB
	 * 
	 * @param file
	 * @return int[] JEg̔z
	 * @throws UnsupportedEncodingException
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public static int[] count(File file) throws UnsupportedEncodingException,
			FileNotFoundException, IOException {

		Counter counter = getConcreteClass(file.getName());
		if (counter == null) {
			return null;
		}

		BufferedReader reader = null;
		int stmtCount = 0;
		int commentCount = 0;
		int blankCount = 0;
		int allCount = 0;

		try {
			reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(file.getPath()), ENCODING), BUFFER_SIZE);
			counter.init();

			String line;
			int result;
			while ((line = reader.readLine()) != null) {
				allCount++;
				result = counter.distinguishLine(line);
				if (result == STATEMENT_LINE) {
					stmtCount++;
				}
				if (result == COMMENT_LINE) {
					commentCount++;
				}
				if (result == BLANK_LINE) {
					blankCount++;
				}
			}

			counter.finish();

		} finally {
			if (reader != null) {
				reader.close();
			}
		}

		return new int[] { stmtCount, commentCount, blankCount, allCount };
	}

	public static String deleteSpace(String str) {
		str = str.replaceAll(" ", "");
		str = str.replaceAll("\t", "");
		return str;
	}

	//---------------------------------------------------------- private
	/**
	 * t@C̊gqɃ}bsOꂽCounter̋`NXԂB 
	 * YNXꍇnullԂB
	 */
	private static Counter getConcreteClass(String fileName) {

		Counter counter = null;
		int lastIndex = fileName.lastIndexOf('.');
		if (lastIndex != -1) {
			String extension = fileName.substring(lastIndex + 1, fileName
					.length());
			counter = (Counter) concreteClassMap.get(extension);
		}

		return counter;
	}

	//------------------------------------------------------- template methods

	/**
	 * Vȃt@CJEgO1xsB 
	 * KvɉăJEgJnȌB
	 */
	protected void init() {
	}

	/**
	 * t@CJEgIƂ1xsB 
	 * KvɉăJEgIƂ̌nB
	 */
	protected void finish() {
	}

	//---------------------------------------------------------- abstract

	/**
	 * œsAssRgss𔻒肵lԂB
	 * 
	 * @param line
	 * @return ss̏ꍇCounter.STATEMENT_LINE <br>
	 *         Rgs̏ꍇCounter.COMMENT_LINE <br>
	 *         s̏ꍇCounter.BLANK_LINEԂ
	 */
	abstract protected int distinguishLine(String line);
}

